@@ -1,18 +1,19 @@
 /*
- * main.c - v1.10 - 25 Nov 2001
- * (c) Stevie Strickland, 1999-2001
+ * main.c - v1.14 - 18 December 2010
+ * (c) Stevie Strickland, 1999-2012
  *
  * This program has been placed under the GPL.  Any bugfixes or enhancements
  * will be greatly appreciated :)
  *
- * Stevie Strickland - sstrickl@cc.gatech.edu
+ * Stevie Strickland - sstrickl@ccs.neu.edu
  */
 
+/* For the abs() function */
+#include <stdlib.h>
+
 #include "rolldice.h"
+#include "version.h"
 
-/* The version number :) */
-static const int MAJOR_VERSION = 1;
-static const int MINOR_VERSION = 10;
 
 /* The long options for this program in an struct option array */
 struct option long_opts[] = {{"help", 0, NULL, 'h'},
@@ -51,8 +52,8 @@ static void print_usage(int exitval) {
  */
 static void print_version() {
     printf("rolldice, v%d.%d\n", MAJOR_VERSION, MINOR_VERSION);
-    printf("Written by Stevie Strickland (sstrickl@resnet.gatech.edu)\n");
-    exit(EXIT_SUCCESS);
+    printf("Written by Stevie Strickland (sstrickl@ccs.neu.edu)\n");
+    exit(EX_OK);
 }
 
 /* print_rolls() - Prints the rolls, either just the totals or the
@@ -64,18 +65,12 @@ static void print_version() {
 void print_rolls(int *dice_nums) {
     int i, j, k, temp_int, temp_index, temp_total;
     int* temp_roll;
-    
-    if(dice_nums == NULL) {
-        fprintf(stderr, "Problems with the dice string, either malformed "
-			"or numbers are too large,\nso quitting!\n");
-	exit(EXIT_FAILURE);
-    }
-    
+     
     if((temp_roll = malloc(sizeof(*temp_roll) * dice_nums[NUM_DICE])) == NULL) {
         perror("rolldice");
-	exit(EXIT_FAILURE);
+    	exit(EX_OSERR);
     }
-
+    
     for(i = 0; i < dice_nums[NUM_ROLLS]; i++) {
 	temp_total = 0;
 	if(print_separate) printf("Roll #%d: (", i+1);
@@ -85,7 +80,7 @@ void print_rolls(int *dice_nums) {
 	    temp_total += temp_roll[j];
 	}
 	for(j = 0; j < dice_nums[NUM_DROP]; j++) {
-	    temp_int = MAXSHORT;
+	    temp_int = SHRT_MAX;
 	    for(k = 0; k < dice_nums[NUM_DICE]; k++)
 		if(temp_int > temp_roll[k]) {
 		    temp_int = temp_roll[k];
@@ -93,7 +88,7 @@ void print_rolls(int *dice_nums) {
 		}
 	    if(print_separate) printf("- %d ", temp_int);
 	    temp_total -= temp_int;
-	    temp_roll[temp_index] = MAXSHORT;
+	    temp_roll[temp_index] = SHRT_MAX;
 	}
 	if(print_separate) printf(") ");
 	if(dice_nums[MULTIPLIER] != 1) {
@@ -101,7 +96,13 @@ void print_rolls(int *dice_nums) {
 	    temp_total *= dice_nums[MULTIPLIER];
 	}
 	if(dice_nums[MODIFIER]) {
-	    if(print_separate) printf("+ %d ", dice_nums[MODIFIER]);
+	    if(print_separate){
+            if (dice_nums[MODIFIER] > 0)
+                printf("+ %d ", dice_nums[MODIFIER]);
+            else
+                printf("- %d ", abs(dice_nums[MODIFIER]));
+
+        }
 	    temp_total += dice_nums[MODIFIER];
 	}
 	if(print_separate) printf("= ");
@@ -111,28 +112,71 @@ void print_rolls(int *dice_nums) {
     if(!print_separate) printf("\n");
 }
 
+/* roll_from_stdin() - parse stdin, one roll by line
+ * 
+ * Parameters: None
+ * Returns: EX_OK or EXIT_DATAERR (if bad stdin)
+ */
+int roll_from_stdin(){
+     int *dice_nums = NULL;
+     static char *line = (char *)NULL;
+
+     line = readline("");
+     while(line){
+       dice_nums = parse_string( line );
+       if ( dice_nums == NULL ) {
+         return EX_DATAERR;
+       }
+       free(line);
+       print_rolls(dice_nums);
+       free(dice_nums);
+
+       line = (char *)NULL;
+       line = readline("");
+     }
+    return EX_OK;
+}
+
+/* roll_from_args() - parse command line args to roll dices
+ * 
+ * Parameters: args passed to CLI
+ * Returns: EX_OK or EXIT_DATAERR (if bad command line)
+ */
+int roll_from_args(char **argv){
+    int *dice_nums = NULL;
+    int index;
+
+    for(index = optind; argv[index] != NULL; index++) {
+      dice_nums = parse_string( argv[index] );
+      if ( dice_nums == NULL ) {
+        return EX_DATAERR;
+      }
+     print_rolls(dice_nums);
+     free(dice_nums);
+    }
+    return EX_OK;
+}
+
+
 int main(int argc, char **argv) {
-  
-    int c, index, *totals, *dice_nums;
-    
-    dice_nums = NULL;
+    int c;
 
     while((c = getopt_long(argc, argv, "hvrus", long_opts, NULL)) != -1) {
       switch(c) {
       case 'h':
-	print_usage(EXIT_SUCCESS); break;
+	print_usage(EX_OK); break;
       case 'v':
 	print_version(); break;
       case 'r':
 	if(rand_file == URANDOM) {
 	  fprintf(stderr, "Choose either '-r' or '-u', please.\n");
-	  return EXIT_FAILURE;
+	  return EX_USAGE;
 	}
 	rand_file = RANDOM; break;
       case 'u':
 	if(rand_file == RANDOM) {
 	  fprintf(stderr, "Choose either '-r' or '-u', please.\n");
-	  return EXIT_FAILURE;
+	  return EX_USAGE;
 	}
 	rand_file = URANDOM; break;
       case 's':
@@ -142,19 +186,10 @@ int main(int argc, char **argv) {
     
     init_random(rand_file);
     
-    if ( argc < 2 ) {
-      print_usage(EXIT_FAILURE);
+    if ( optind == argc ) {
+      return roll_from_stdin();
     }
-
-    for(index = optind; argv[index] != NULL; index++) {
-      dice_nums = parse_string( argv[index] );
-      if ( dice_nums == NULL ) {
-	fprintf(stderr, "%s: Failure in getting dice attributes\n", argv[0]);
-	return EXIT_FAILURE;
-      }
-      print_rolls(dice_nums);
-      free(dice_nums);
+    else {
+      return roll_from_args(argv);
     }
-    
-    return EXIT_SUCCESS;
 }
